Package: rODE
Type: Package
Title: Ordinary Differential Equation (ODE) Solvers Written in R Using
        S4 Classes
Version: 0.99.6
Authors@R: person("Alfonso R.", "Reyes", role = c("aut", "cre"), email = "alfonso.reyes@oilgainsanalytics.com")
Description: Show physics, math and engineering students how an ODE solver
    is made and how effective R classes can be for the construction of
    the equations that describe natural phenomena. Inspiration for this work 
    comes from the book on "Computer Simulations in Physics" 
    by Harvey Gould, Jan Tobochnik, and Wolfgang Christian. 
    Book link: <http://www.compadre.org/osp/items/detail.cfm?ID=7375>.
Depends: R (>= 3.3.0)
License: GPL-2
Encoding: UTF-8
Imports: methods, data.table
LazyData: true
Suggests: knitr, testthat, rmarkdown, ggplot2, dplyr, tidyr, covr,
        scales
RoxygenNote: 6.0.1
Collate: 'ode_generics.R' 'ODESolver.R' 'ODE.R' 'AbstractODESolver.R'
        'ODEAdaptiveSolver.R' 'DormandPrince45.R' 'Euler.R'
        'EulerRichardson.R' 'ODESolverFactory.R' 'RK4.R' 'RK45.R'
        'Verlet.R' 'rODE-package.r' 'utils.R'
VignetteBuilder: knitr
URL: https://github.com/f0nzie/rODE
NeedsCompilation: no
Packaged: 2017-11-10 01:31:57 UTC; msfz751
Author: Alfonso R. Reyes [aut, cre]
Maintainer: Alfonso R. Reyes <alfonso.reyes@oilgainsanalytics.com>
Repository: CRAN
Date/Publication: 2017-11-10 04:17:51 UTC
Built: R 4.6.0; ; 2026-04-27 02:13:41 UTC; unix
