# Generated by roxygen2: do not edit by hand

export("init<-")
export("setSolver<-")
export("setTolerance<-")
export(AbstractODESolver)
export(DormandPrince45)
export(Euler)
export(EulerRichardson)
export(ODE)
export(ODESolver)
export(ODESolverFactory)
export(RK4)
export(RK45)
export(Verlet)
export(doStep)
export(enableRuntimeExceptions)
export(getEnergy)
export(getErrorCode)
export(getExactSolution)
export(getODE)
export(getRate)
export(getRateCounter)
export(getRateCounts)
export(getState)
export(getStepSize)
export(getTime)
export(getTolerance)
export(importFromExamples)
export(init)
export(run_test_applications)
export(setState)
export(setStepSize)
export(setTolerance)
export(showMethods2)
export(step)
exportClasses(ODEAdaptiveSolver)
exportClasses(ODESolverFactory)
exportClasses(RK4)
exportMethods(AbstractODESolver)
exportMethods(DormandPrince45)
exportMethods(Euler)
exportMethods(EulerRichardson)
exportMethods(ODEAdaptiveSolver)
exportMethods(ODESolverFactory)
exportMethods(RK4)
exportMethods(Verlet)
exportMethods(createODESolver)
importFrom(methods,callNextMethod)
importFrom(methods,new)
importFrom(methods,showMethods)
